@@ -1,7 +1,7 @@
 print.bivden <- function(x, ...){
-	cat("Bivariate kernel density estimate\n\n")
+	cat("Bivariate kernel density/intensity estimate\n\n")
 	if(length(unique(x$h))==1) cat("Fixed isotropic smoothing with h =",unique(x$h[!is.na(x$h)]),"unit(s)\n")
 	else cat("Adaptive isotropic smoothing with (pilot) h =",x$pilotH,"global h =",x$globalH,"unit(s)\n")
 	
-	cat("No. of observations:",nrow(x$data),"\n")
+	cat("No. of observations:",sum(x$counts),"\n")
 }
\ No newline at end of file
